#include "variables.h"

double RelHumidity(struct dataProf Met,struct dataInit Init,aryDpSpTp Mp, double BaseT, double BaseLWC)
{
 double rhum,Le,T,Td;
T=Met.temp;		Td=Met.dewt;
/*Le=597.3*pow((ZEROC/T),(0.167+3.67e-4*T))*JperCAL; 
 ITcal/g -> J/g specific latent heat from Pruppacher and Klett
rhum=exp((Td-T)/(RGAS*MolWt(h2oSP)*NM2perMBAR*T*(Td/Le)));*/
rhum=h2oSatWv(Td)/h2oSatWv(T);
if (Met.alt>=Init.base-10){rhum = 1.0;}
if (rhum > 1.000) {rhum=1.0000;}
if (Met.alt>=Init.base) {   /* in cloud specification */
  rhum=(h2oSatWv(BaseT)-LiqWaterContent(Mp)+BaseLWC)/h2oSatWv(T);/*baseLWC accounts for small amount of liquid water below cloud base*/
printf("BaseT %e BaseLWC %e rhum %e\n", BaseT, BaseLWC, rhum);
printf("temp %e rhum %e h2oSatWv(T) %e\n",T, rhum, h2oSatWv(T));
  /*rhum=1.+(ScMAX*((Init.maxlwc-LiqWaterContent(Mp))/Init.maxlwc)*/
/*   *((Init.height-Met.alt)/(Init.height-Init.base))  */
  /*((Met.alt-Init.base)/(Init.height-Init.base)));*/
  if (rhum < 1.000) {rhum=1.0;}}

return rhum;
}

double LiqWaterContent(aryDpSpTp Mp)
/* g/cc */
{
 double mass=0;
 int i,k;
for (i=0;i<DpDIM;i++) {for (k=0;k<TpDIM;k++) {mass+=Mp[i][h2oSP][k];}}
return mass;
}

double h2oSatWv(double T)
/*g/cm3 Saturated water vapor content, from correlation in Pruppacher and 
Klett, p. 350*/
{
 double h2oSatVp,satwv;
T-=ZEROC; /*K to deg. C*/
h2oSatVp=6.107799961+T*(4.436518521e-1+T*(1.428945805e-2+T*(2.650648471e-4 
 +T*(3.031240396e-6+T*(2.034080948e-8+6.136820929e-11*T))))); /*esat in mb*/
satwv=h2oSatVp*MolWt(h2oSP)/(RGAS*(T+ZEROC)*CCperM3); /*g/cm3*/
return satwv;
}

double min(double Dp1, double Dp2)
{
if (Dp1<Dp2) return Dp1;
else return Dp2;
}

double max(double Dp1, double Dp2)
{
if (Dp1>Dp2) return Dp1;
else return Dp2;
}

double dryDp(float NUMsect)
{
double diam;
diam=DpMIN*exp(((NUMsect/(DpDIM-1))*LN(DpMAX/DpMIN)));
return diam;
}

double AirMeanPath (struct dataProf Met)
{
 double lambda;
lambda=0.065e-4; /* cm at 298K p. 317, Seinfeld 1986 */
return lambda;
}

double AirViscosity (struct dataProf Met)
{
 double mu;
mu=1.83e-4; /*poise (g/cm/sec) at 298K p. 317, Seinfeld 1986 ; CRC F-150*/
return mu;
}

double wetPMass(int Dp, int Tp,double h2oMp, aryDpSpTp Mp,double Np)
/* Calculates the total mass of all species in one PARTICLE 
   Units are g/particle */
{
 int j;			double mass=0.0;
if (Np>zeroTHRESH) {
  for (j=0;j<SpDIM;j++) {
    if (!((j==dmsSP)|(j==so2SP)|(j==h2oSP))) {
      mass+=(Mp[Dp][j][Tp]/Np);}}
  mass+=(h2oMp/Np);}
else {mass=0;/*SpDensity(h2oSP)*pow(dryDp(Dp),3.0)*PI/6.0;/**/}
if (mass>1.) {printf("PMass i%dk%d %e %e %e %e %e \n",Dp,Tp,mass,h2oMp,Np,Mp[Dp][0][Tp],Mp[Dp][5][Tp]);}
return mass;
}

double wetSMass(int Dp, int Tp, double h2oMp,aryDpSpTp Mp)
/* Calculates the total mass of all species in a SECTION
   Units are g */
{
 int j;			double mass=0;
mass=h2oMp;
for (j=0;j<SpDIM;j++) {
  if (!((j==dmsSP)|(j==so2SP)|(j==h2oSP))) {
    mass+=Mp[Dp][j][Tp];}}
return mass;
}

double wetPDensity (int Dp, int Tp, double h2oMp,aryDpSpTp Mp)
/* g/um3 Calculates weighted mean particle density.
   Mixing rule from Cohen et al.,1987:eqn. 13*/
{
 int j;			double density=0,Vmix=0,TotVol=0,TotMass=0;
TotMass=h2oMp;
TotVol=h2oMp/SpDensity(h2oSP);
if (!(drySMass(Dp,Tp,Mp)==0)) {
  for (j=0;j<SpDIM;j++) {
    if (!((j==dmsSP)|(j==so2SP)|(j==h2oSP))) {
      TotMass+=Mp[Dp][j][Tp];
      TotVol+=(Mp[Dp][j][Tp]/SpDensity(j));}}
  density=TotMass/TotVol;
  /*if (density<SpDensity(h2oSP)) {density=SpDensity(h2oSP);}/**/}
else {density=SpDensity(h2oSP);}
return density;
}

double wetDp(int Dp, int Tp, double h2oMp,aryDpSpTp Mp,double Np)
/* um: Compute the particle/droplet diameter at the ambient humidity
   and temperature.  This diameter is the diameter used to calculate
   surface areas, etc., for condensation, coagulation, deposition. */
{
 double diam,density;
density=wetPDensity(Dp,Tp,h2oMp,Mp);
/*if (density >=1.) {diam=dryDp(Dp);}/**//*if density>1e12g/cc, then no mass in sec*/
/*else {/**/ 
if (wetPMass(Dp,Tp,h2oMp,Mp,Np)>0) {
  diam=pow( (6.*wetPMass(Dp,Tp,h2oMp,Mp,Np)/(PI*density)), (1./3.) );}
  else {diam=dryDp(Dp);}/*}*/
if (diam>3.e2) {printf("Diam %e %e %e %e %e\n",diam,dryDp(Dp),density,wetPMass(Dp,Tp,h2oMp,Mp,Np),drySMass(Dp,Tp,Mp));}
return diam;
}

double SpDensity (int Sp)
/*g/um3 Species condensed phase density from CRC #71*/
{
 double density;
switch (Sp) {
  case h2oSP : {density=1.0*pow(CMperMIC,3.); break;}
  case dmsSP : {density=1.e5; break;} /* DMS is a gas-phase species only */
  case so2SP : {density=1.841*pow(CMperMIC,3.); break;}
  case h2so4SP : {density=1.841*pow(CMperMIC,3.); break;} 
  case blkCSP : {density=2.25*pow(CMperMIC,3.); break;} 
  case naclSP : {density=2.165*pow(CMperMIC,3.); break;} 
  case orgCSP : {density=0.8*pow(CMperMIC,3.); break;}
  case nh42so4SP:{density=1.769*pow(CMperMIC,3.); break;}
  case nh4hso4SP:{density=1.78*pow(CMperMIC,3.); break;}
  case airSP: {density= 1.29e3/pow(MICperM,3.); break;}
  case nh4no3SP : {density = 1.725*pow(CMperMIC,3.);break;}
  case nano3SP : {density=2.261*pow(CMperMIC,3.);break;}  
  case hno3SP : {density = 1.503*pow(CMperMIC,3.);break;}
default : {density=1.e5; break;}
}
return density;
}

double MolWt (int Sp)
/*g/mole Molecular Weight of species  from CRC #71*/
{
 double weight;
switch (Sp) {
  case h2oSP : {weight=18.0; break;}
  case dmsSP : {weight=62.; break;} /* DMS is a gas-phase species only */
  case so2SP : {weight=64.; break;} 
  case h2so4SP : {weight=98.07; break;}
  case blkCSP : {weight=12.011; break;} 
  case naclSP : {weight=58.44; break;} 
  case orgCSP : {weight=144; break;} /* ~octacosane/octacosanoic acid */
  case nh42so4SP:{weight=132.13; break;}
  case nh4hso4SP:{weight=115.10; break;}
  case airSP: {weight= 28.965; break;}
  case nh4no3SP : {weight = 80.04; break;}
  case nano3SP : {weight = 84.99; break;}
  case hno3SP : {weight = 63.01; break;}
  default : {weight=0.; break;}
}
return weight;
}

double SeaSaltDensity ()
/*density of sea salt in g/um3, based on mass fraction of NaCl
in the solution */
{
 double 	density=1.e-12,Vmix=0,TotVol=0,h2oFrac;
h2oFrac=1.-naclFRAC; /*neglect Ca, Fl, etc. */
TotVol=(naclFRAC/SpDensity(naclSP))+(h2oFrac/SpDensity(h2oSP));
density=1./TotVol;
return density;
}

double dryPDensity (int Dp, int Tp, aryDpSpTp Mp)
/* g/um3 Calculates weighted mean particle density for dry particles.
  Density from Cohen et al., 1987, eqn. 13*/
{
 int j;		double density=1.e-12,TotVol=0,TotMass=0;
 FILE            *GrowthFilePtr;
 word24          GrowthFileName;
strcpy(GrowthFileName,"Growth");
strcat(GrowthFileName,OUTsuffix);
if (!(drySMass(Dp,Tp,Mp)==0)) {

  for (j=0;j<SpDIM;j++) {if (!((j==dmsSP))) {
    if (!(j==h2oSP)) {
      TotMass+=Mp[Dp][j][Tp];
      TotVol+=( Mp[Dp][j][Tp]/SpDensity(j) );}}}
  density=TotMass/TotVol;
  /*if (density<SpDensity(h2oSP)) {density=SpDensity(h2oSP);}*/}
else {density=SpDensity(h2oSP);}  
if (!(density>0)) {
  density=SpDensity(h2oSP);
  GrowthFilePtr=fopen(GrowthFileName,"a");
  fprintf(GrowthFilePtr,"density not >0, %e %e %e\n",density,TotMass,TotVol);}
return density;
}


double dryPMass(int Dp, int Tp, aryDpSpTp Mp)
/* Calculates the mass per particle based on the given composition and
   diameter of all species EXCEPT WATER.
   Units are g/particle */
{
 int j; 		double mass=0;
mass=dryPDensity(Dp,Tp,Mp)*pow(dryDp(Dp),3.)*PI/6.;
return mass;
}

double drySMass(int Dp, int Tp, aryDpSpTp Mp)
/* Calculates the total mass of all species EXCEPT WATER in a SECTION
   Units are g */
{
 int j;			double mass=0;
for (j=0;j<SpDIM;j++) {if (!(j==h2oSP)) {
  if (!((j==dmsSP))) {mass+=Mp[Dp][j][Tp];}}}
return mass;
}

double Surface(int Dp, int Tp,double h2oMp, aryDpSpTp Mp,double Np, struct dataProf Met)
/* Surface tension term, dimensionless, ln-form */
{
 double	diam, rhop,sigmaWA,sigmaSA,surface;


diam=wetDp(Dp,Tp,h2oMp,Mp,Np)/MICperM;
rhop=wetPDensity(Dp,Tp,h2oMp,Mp)*pow(CMperMIC,-3.0); /*g/cc*/
sigmaWA=76.10-0.155*(Met.temp-ZEROC);/*p. 130 P&K dyn/cm, T in C*/
sigmaSA=sigmaWA;
if (h2oMp>0) {
  if (Mp[Dp][naclSP][Tp]>0) 
   {sigmaSA+=( 27.0*Mp[Dp][naclSP][Tp]/h2oMp );}/*dyne/cm for solution*/
  if (Mp[Dp][nh42so4SP][Tp]>0) 
   {sigmaSA+=( 16.4*Mp[Dp][nh42so4SP][Tp]/h2oMp );}
  if (Mp[Dp][nh4hso4SP][Tp]>0) 
   {sigmaSA+=( 18.8*Mp[Dp][nh4hso4SP][Tp]/h2oMp );}
  if (Mp[Dp][nano3SP][Tp]>0)
   {sigmaSA+=( 10.8*Mp[Dp][nano3SP][Tp]/h2oMp);}
  if (Mp[Dp][nh4no3SP][Tp]>0)
   {sigmaSA+=( 10.8*Mp[Dp][nh4no3SP][Tp]/h2oMp);}}

surface=(4.*(MolWt(h2oSP)*sigmaSA*CMperM/GperKG)
 /(RGAS*CCperM3*NM2perMBAR*(Met.temp)*rhop*diam*CMperM)); /*P&K p.420*/
printf("surfacet %e sigmaSA %e temp %e rhop %e diam %e", surface, sigmaSA, Met.temp, rhop, diam);
return surface;
}

double Equilibrium(double MassWater,double h2oCv,int Dp,int Tp,aryDpSpTp Mp,double Np, struct dataProf Met)
/*Equilibrium=(RelativeHumidity-100)/100 + exp(Surface-Activity) 
at Equilibrium=0 the particles contain the appropriate mass of water.
If Equilibrium>0, particles don't have enough LWC for the ambient RH.
If Equilibrium<0, particles have too much LWC for ambient RH.*/
{
 double equilibrium=0.1;
equilibrium=(h2oCv/h2oSatWv(Met.temp))-
 exp(Surface(Dp,Tp,MassWater,Mp,Np,Met)-Activity(h2oSP,Tp,Dp,MassWater,Mp,Np,Met));
return equilibrium;
}

double Activity(int Sp,int Tp,int Dp, double h2oMp,aryDpSpTp Mp, double Np, struct dataProf Met)
/*Pitzer method Zemaitis, p. 241 */
{
 double OsmCoef,activity=1.0,imc[IpcDIM],ima[IpaDIM],A,f,aGamma,cGamma;
 double imSum=0,I=0,mzSum=0, mzSum2, mzcSum=0,mzaSum=0,thetaSum=0,psiSum=0,mmBCSum=0;
 double mBCSum=0,mmpsiSum=0,mmBpCSum=0,mzOrg=0;
 double	B[IpcDIM][IpaDIM],Bp[IpcDIM][IpaDIM],zc[IpcDIM],za[IpaDIM];
 double beta0[IpcDIM][IpaDIM],beta1[IpcDIM][IpaDIM],C[IpcDIM][IpaDIM];
 double thetaa[IpaDIM][IpaDIM],psia[IpaDIM][IpaDIM][IpcDIM];
 double thetac[IpcDIM][IpcDIM],psic[IpcDIM][IpcDIM][IpaDIM];
 double Sol = .03, molorgh2o, nondis;
 double orgMw;
 int 	j,a,c,a1,c1,a2,c2; /*z charge on ion */
 arySp 	mm; /*mm molec. molality, im ion molality */



for (c1=0;c1<IpcDIM;c1++) {
  for (a1=0;a1<(IpaDIM);a1++) {
    beta0[c1][a1]=0;	beta1[c1][a1]=0;	C[c1][a1]=0;
    for (c2=0;c2<IpcDIM;c2++) {thetac[c1][c2]=0; psic[c1][c2][a1]=0;}
    for (a2=0;a2<IpaDIM;a2++) {thetaa[a1][a2]=0; psia[a1][a2][c1]=0;} }}
/*Zemaitis data*/
beta0[naIP][clIP] = 0.0765+7.159e-4*(Met.temp-TREF); 	
beta1[naIP][clIP] = 0.2664+7.005e-4*(Met.temp-TREF);
C[naIP][clIP] =0.00127-10.54e-5*(Met.temp-TREF); 

beta0[nh4IP][clIP] = 0.0522+0.779e-4*(Met.temp-TREF); 	
beta1[nh4IP][clIP] = 0.1918+12.58e-4*(Met.temp-TREF);
C[nh4IP][clIP] =-0.00301-2.10e-5*(Met.temp-TREF); 

beta0[naIP][so4IP] = (0.0261+3.156e-3*(Met.temp-TREF))*(3./4.); 	
beta1[naIP][so4IP] = (1.484+7.51e-3*(Met.temp-TREF))*(3./4.);
C[naIP][so4IP] =(0.01075-9.2e-4*(Met.temp-TREF))*(3./pow(2.,2.5)); 

beta0[nh4IP][so4IP] = (0.0545+0.e-3*(Met.temp-TREF))*(3./4.); 	
beta1[nh4IP][so4IP] = (0.878+0.e-3*(Met.temp-TREF))*(3./4.);
C[nh4IP][so4IP] =(-0.00219-0.e-4*(Met.temp-TREF))*(3./pow(2.,2.5)); 

psia[clIP][so4IP][naIP]=0.0014;

beta0[nh4IP][no3IP] = -0.0154;
beta1[nh4IP][no3IP] = 0.1120;
C[nh4IP][no3IP] = -0.00003;

beta0[naIP][no3IP] = 0.0068+(12.66e-4)*(Met.temp-TREF);
beta1[naIP][no3IP] = 0.1783+(20.6e-4)*(Met.temp-TREF);
C[naIP][no3IP] = -0.000072-(23.1e-5)*(Met.temp-TREF);

beta0[hIP][no3IP] = 0.1119;
beta1[hIP][no3IP] = 0.3206;
C[hIP][no3IP] = 0.0010;

/*Pitzer 1991, p. 100, 101,109,118,322 */
beta0[naIP][ohIP]=0.0864+7.e-4*(Met.temp-TREF);
beta1[naIP][ohIP]=0.253+1.34e-4*(Met.temp-TREF);
C[naIP][ohIP]=0.0044-18.9e-5*(Met.temp-TREF);

beta0[naIP][hso4IP]=0.0454;
beta1[naIP][hso4IP]=0.398;

beta0[hIP][hso4IP]=0.2106+48.01*((1./Met.temp)-(1./TREF));
beta1[hIP][hso4IP]=0.5320+1183.*((1/Met.temp)-(1./TREF))+2.364*LN(Met.temp/TREF);

beta0[hIP][so4IP]=0.0217+58.33*((1./Met.temp)-(1./TREF));
C[hIP][so4IP]=0.0411+58.65*((1./Met.temp)-(1./TREF));

beta0[hIP][clIP]=0.1775-3.08e-4*(Met.temp-TREF);
beta1[hIP][clIP]=0.2945+1.41e-4*(Met.temp-TREF);
C[hIP][clIP]=0.00080-6.21e-5*(Met.temp-TREF);

thetac[naIP][nh4IP]=0;
psic[naIP][nh4IP][clIP]=-0.0003;
psic[naIP][nh4IP][so4IP]=-0.0013;
psic[naIP][nh4IP][hso4IP]=0;
thetac[naIP][hIP]=0.036;
psic[naIP][hIP][clIP]=-0.004;
psic[naIP][hIP][hso4IP]=-0.0129;
thetaa[clIP][so4IP]=0.03;
thetaa[clIP][hso4IP]=-0.006;
psia[clIP][hso4IP][naIP]=-0.006;
psia[clIP][hso4IP][hIP]=0.013;
thetaa[clIP][ohIP]=-0.050;
psia[clIP][ohIP][naIP]=-0.006;
psia[so4IP][hso4IP][naIP]=-0.0094;
thetaa[ohIP][so4IP]=-0.013;
psia[so4IP][ohIP][naIP]=-0.009;
thetac[nh4IP][hIP]=-0.0127;
psic[nh4IP][hIP][clIP]=-.0091;

psia[clIP][no3IP][naIP]= -0.006;
thetaa[clIP][no3IP]= 0.016;

if (h2oMp>zeroTHRESH) {
for (j=0;j<SpDIM;j++) {
 mm[j]=(Mp[Dp][j][Tp]/MolWt(j))/(h2oMp/GperKG);}
za[so4IP]=2.;	ima[so4IP]=mm[nh42so4SP];
zc[nh4IP]=1.;	imc[nh4IP]=mm[nh4hso4SP]+2.*mm[nh42so4SP]+mm[nh4no3SP];
zc[naIP]=1.;	imc[naIP]=mm[naclSP]+mm[nano3SP];
za[clIP]=1.;	ima[clIP]=mm[naclSP];
za[no3IP]=1.;   ima[no3IP]=mm[nh4no3SP]+mm[nano3SP]+mm[hno3SP];
zc[hIP]=1.;	imc[hIP]=pow(10.,-1.92)/so4hso4FRAC+mm[hno3SP];
 /*pKa=1.92; Ka=[H+][SO4--]/[HSO4-]=[H+]*so4hso4FRAC, Atkins p. 277*/
za[ohIP]=1.;	ima[ohIP]=1.008e-14/imc[hIP];
za[hso4IP]=1.;	ima[hso4IP]=mm[nh4hso4SP];

for (c=0;c<IpcDIM;c++) {     
  imSum+=imc[c]; 
  mzcSum+=( imc[c]*zc[c] );
  I+=( 0.5*imc[c]*zc[c]*zc[c] );}  /*p.236*/
for (a=0;a<IpaDIM;a++) {     
  imSum+=ima[a];   
  mzaSum+=( ima[a]*za[a] ); 
  I+=( 0.5*ima[a]*za[a]*za[a] );}

if (Sp==h2oSP) {
/*Pitzer method Zemaitis, p. 241 */
  A=0.3915+(0.006*(Met.temp-TREF)/10.0);    /*(kg/mole)^0.5  eqn. 4.64*/
   /*p.99 Pitzer 1991: 0.3915@25C; temp dependence -.006/10 deg*/
  f=-A*pow(I,0.5)/(1.0+1.2*pow(I,0.5));
  mzSum=0.5*(mzcSum+mzaSum);
  for (c1=0;c1<IpcDIM;c1++) {     
    for (c2=0;c2<IpcDIM;c2++) {
      thetaSum+=thetac[c1][c2];
      for (a=0;a<(IpaDIM);a++) {psiSum+=( ima[a]*psic[c1][c2][a] ); }}}
  for (a1=0;a1<IpaDIM;a1++) {     
    for (a2=0;a2<IpaDIM;a2++) {
      thetaSum+=thetaa[a1][a2];
      for (c=0;c<(IpcDIM);c++) {psiSum+=( imc[c]*psia[a1][a2][c] ); }}}
  for (c=0;c<IpcDIM;c++) {
    for (a=0;a<(IpaDIM);a++) {
      B[c][a]=beta0[c][a]+(beta1[c][a]*exp(-2.*pow(I,0.5)));
      mmBCSum+=(imc[c]*ima[a]*(B[c][a]+(mzSum*C[c][a]/pow((zc[c]*za[a]),0.5))));}}
  OsmCoef=1.+((1./(imSum))*(2.*I*f+2.*mmBCSum+thetaSum+psiSum));
  if (OsmCoef<0) {OsmCoef=1.0;}
  activity=OsmCoef*MolWt(h2oSP)*mzSum/GperKG;
/*IDEALLY*/ 
/*  activity=-((GperKG/MolWt(h2oSP))/mzSum)
   *LN((GperKG/MolWt(h2oSP))/(GperKG/MolWt(h2oSP)+mzSum));
  activity=exp(LN(GperKG/MolWt(h2oSP)/(GperKG/MolWt(h2oSP)+mzSum)));
  activity=mzSum*MolWt(h2oSP)/GperKG;*/
/* for organics add correct number of ions to mzSum, no change to OsmCoef*/


orgMw = MolWt(orgCSP);

if (Tp>0.5){orgMw = 144;}

if (h2oMp<drySMass(Dp, Tp, Mp)){h2oMp = drySMass(Dp,Tp,Mp);}
  if (SpDIM>7) { mzOrg=(Mp[Dp][orgCSP][Tp]/MolWt(orgCSP))/(h2oMp/GperKG);
/*    if ((Mp[Dp][orgCSP][Tp]-(Sol*h2oMp/(100-Sol)))> 0){
      activity=OsmCoef*MolWt(h2oSP)*(mzSum/GperKG /*+mzOrg*/ 
/*      +((Mp[Dp][orgCSP][Tp]/MolWt(orgCSP))/h2oMp)
      -(Mp[Dp][orgCSP][Tp]-(Sol*h2oMp/(100-Sol))/MolWt(h2oSP)/SpDensity(orgCSP)/(h2oMp/SpDensity(h2oSP))));
molorgh2o = ((Mp[Dp][orgCSP][Tp]/MolWt(orgCSP))/h2oMp);
nondis = (Mp[Dp][orgCSP][Tp]-(Sol*h2oMp/(100-Sol))/MolWt(h2oSP)/SpDensity(orgCSP)/(h2oMp/SpDensity(h2oSP)));
     printf("h2oMp %e OrgMp %e Sol %e molratio %e notdisolved %e \n", h2oMp, Mp[Dp][orgCSP][Tp], Sol, molorgh2o, nondis);} 
    else{*/

mzSum2 = (3*Mp[Dp][3][Tp]/MolWt(3)+ 3*Mp[Dp][4][Tp]/MolWt(4)+3*Mp[Dp][5][Tp]/MolWt(5)+2*Mp[Dp][6][Tp]/MolWt(6)+ 2*Mp[Dp][9][Tp]/MolWt(9)+2*Mp[Dp][10][Tp]/MolWt(10)+2*Mp[Dp][11][Tp]/MolWt(11))/(h2oMp/GperKG);
      activity=OsmCoef*MolWt(h2oSP)*(mzSum2 /*+mzOrg*/
      +((Mp[Dp][orgCSP][Tp]/orgMw)/(h2oMp/GperKG)))/GperKG;/*}*//*6-31 P&K*/
/*	ReportAct(Dp, wetDp(Dp,Tp,h2oMp,Mp,Np), activity, mzSum, mzSum2, OsmCoef, mzOrg, h2oMp, Met.alt);*/


printf("activity drydp %e i%dj%dk%d h2so4 Mw %e Mp %e org Mw %e Mp %e h2o Mw %e \n", dryDp(Dp),Dp, Sp, Tp, MolWt(5), Mp[Dp][5][Tp], MolWt(7), Mp[Dp][7][Tp], MolWt(h2oSP));

  }
}

if (Sp==h2so4SP) {
/*Pitzer method for SO4 ion, Zemaitis, p. 239 */
  A=0.3915+0.006*(Met.temp-TREF)/10.0;    /*(kg/mole)^0.5  eqn. 4.64*/
   /*p.99 Pitzer 1991: 0.3915@25C; temp dependence -.006/10 deg*/
  f=-A*(pow(I,0.5)/(1.0+1.2*pow(I,0.5))+ (2.0/1.2)*LN(1.0+1.2*pow(I,0.5)));
/*printf("Act-h2so4SP");*/
/*Anion=so4 */
  for (c1=0;c1<IpcDIM;c1++) {     
    for (c2=0;c2<IpcDIM;c2++) {
      mmpsiSum+=( 0.5*imc[c1]*imc[c2]*psic[c1][c2][so4IP] );}}
  for (a=0;a<IpaDIM;a++) {    
    thetaSum+=( ima[so4IP]*2*thetaa[a][so4IP] );
    for (c=0;c<(IpcDIM);c++) {psiSum+=( ima[a]*imc[c]*psia[a][so4IP][c] ); }}
  for (c=0;c<IpcDIM;c++) {
    B[c][so4IP]=beta0[c][so4IP]+(2.0*beta1[c][so4IP]/(4.0*I))
     *(1.0-(1.0+2.0*pow(I,0.5))*exp(-2.0*pow(I,0.5)));
    mBCSum+=( imc[c]*(2.0*B[c][so4IP]+2*mzaSum*C[c][so4IP]) );
    for (a=0;a<IpaDIM;a++) {
      Bp[c][a]=(2.0*beta1[c][a]/(4.0*I*I))
       *(-1.0+(1.0+2.0*pow(I,0.5)+0.5*4.0*I)*exp(-2.0*pow(I,0.5)));
      mmBpCSum+=( imc[c]*ima[a]*za[so4IP]*(za[so4IP]*Bp[c][a]+C[c][a]) );}}
  aGamma=exp(f*pow(za[so4IP],2.0)+mBCSum+thetaSum+psiSum+mmBpCSum+mmpsiSum);
/*Cation=h */
  mmpsiSum=0;thetaSum=0;mBCSum=0;psiSum=0;mmBpCSum=0;
  for (a1=0;a1<IpaDIM;a1++) {     
    for (a2=0;a2<IpaDIM;a2++) {
      mmpsiSum+=( 0.5*ima[a1]*ima[a2]*psia[a1][a2][hIP] );}}
  for (c=0;c<IpcDIM;c++) {    
    thetaSum+=( imc[hIP]*2.*thetac[c][hIP] );
    for (a=0;a<(IpaDIM);a++) {psiSum+=( imc[c]*ima[a]*psic[c][hIP][a] ); }}
  for (a=0;a<IpaDIM;a++) {
    B[hIP][a]=beta0[hIP][a]+(2.0*beta1[hIP][a]/(4.0*I))
     *(1.0-(1.0+2.0*pow(I,0.5))*exp(-2.0*pow(I,0.5)));
    mBCSum+=( ima[a]*(2.0*B[hIP][a]+2.*mzaSum*C[hIP][a]) );
    for (a=0;a<IpaDIM;a++) {
      Bp[c][a]=(2.0*beta1[c][a]/(4.0*I*I))
       *(-1.0+(1.0+2.0*pow(I,0.5)+0.5*4.0*I)*exp(-2.0*pow(I,0.5)));
      mmBpCSum+=( imc[c]*ima[a]*zc[hIP]*(zc[hIP]*Bp[c][a]+C[c][a]) );}}
 cGamma=exp(f*pow(zc[hIP],2.0)+mBCSum+thetaSum+psiSum+mmBpCSum+mmpsiSum);
 activity=pow((pow(aGamma,za[so4IP])*pow(cGamma,zc[hIP])),
   (1./(za[so4IP]+zc[hIP])))*ima[so4IP];
}
}
else { 
  if (Sp==h2oSP) {activity=0.;}/*if no water present, set water activity=0*/
  else {activity=1.0;}} /*if no water present, set other activities=1*/
/*if ((Sp==h2oSP)&&(mzSum>1.0)&&(activity>1.0)) {activity=1.0;}*/
/*if (activity > 1.0) {activity=1.0;}*/
/*if (activity > 0.086) {activity=0.086;}*/
printf("Act11%e i%dj%dk%d h2o%e h2so4%e \n", activity,Dp,Sp,Tp,h2oMp,Mp[Dp][nh42so4SP][Tp]); 

return activity;
}
